#include "MyGLWidget.hpp"

#include "GLExtensions.hpp"

#include <cat/AllGraphics.hpp>
using namespace cat;

MyGLWidget::MyGLWidget(QWidget *parent)
    : QGLWidget(parent)
{
    startTimer(15); // shooting for 60 FPS
}

MyGLWidget::~MyGLWidget()
{
}

QSize MyGLWidget::minimumSizeHint() const
{
    return QSize(640, 640);
}

QSize MyGLWidget::sizeHint() const
{
    return QSize(800, 800);
}

void MyGLWidget::initializeGL()
{
    // Orthographic projection
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    glOrtho(-3, 3, 3, -3, -4, 15);

    // Enable smooth color gradients between vertices
    glShadeModel(GL_SMOOTH);

    // Identity modelview
    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();

    // Background color is black
    glClearColor(0.0f, 0.0f, 0.0f, 0.0f);

    // Enable the depth test to cause faces to render from back to front
    glEnable(GL_DEPTH_TEST);
    glDepthFunc(GL_LEQUAL);

    // Use the nicest perspective correction hint
    glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);

    // Enable cull face so faces with normals pointing away from the camera
    // do not get rendered
    glEnable(GL_CULL_FACE);

    angle = 0.0f;
}

void MyGLWidget::paintGL()
{
    // Clear color (background) and depth (for depth test) buffers
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

    glMatrixMode(GL_MODELVIEW);

    Quaternion4d q;
    q.setFromEulerAngles(Deg2Rad(30.0), 0, Deg2Rad(angle));

    Matrix4x4d m;
    q.getMatrix(m);

    glLoadMatrixd(&m(0));

    glTranslatef(-1.5f, 0.0f, 0.0f);

    makeCube();

    glTranslatef(3.0f, 0.0f, 0.0f);

    makeCube();
}

void MyGLWidget::makeCube()
{
    // Produce a cube by creating a quad strip to make a box with two open ends
    // and then fill in the ends with two quads (saves 6 glVertex3f calls)

    glBegin(GL_QUAD_STRIP);

        glColor3ub(255, 0, 0);
        glVertex3f(1, -1, 1);
        glColor3ub(0, 255, 0);
        glVertex3f(-1, -1, 1);

        glColor3ub(0, 0, 255);
        glVertex3f(1, 1, 1);
        glColor3ub(255, 0, 255);
        glVertex3f(-1, 1, 1);

        glColor3ub(0, 255, 255);
        glVertex3f(1, 1, -1);
        glColor3ub(255, 255, 0);
        glVertex3f(-1, 1, -1);

        glColor3ub(255, 255, 255);
        glVertex3f(1, -1, -1);
        glColor3ub(127, 0, 0);
        glVertex3f(-1, -1, -1);

        glColor3ub(0, 127, 0);
        glVertex3f(1, -1, 1);
        glColor3ub(0, 0, 127);
        glVertex3f(-1, -1, 1);

    glEnd();

    glBegin(GL_QUADS);

        glColor3ub(127, 0, 127);
        glVertex3f(1, -1, -1);
        glColor3ub(127, 127, 0);
        glVertex3f(1, -1, 1);
        glColor3ub(0, 127, 127);
        glVertex3f(1, 1, 1);
        glColor3ub(127, 127, 127);
        glVertex3f(1, 1, -1);

        glColor3ub(0, 0, 0);
        glVertex3f(-1, 1, -1);
        glColor3ub(255, 127, 0);
        glVertex3f(-1, 1, 1);
        glColor3ub(255, 0, 127);
        glVertex3f(-1, -1, 1);
        glColor3ub(255, 127, 127);
        glVertex3f(-1, -1, -1);

    glEnd();
}

void MyGLWidget::timerEvent(QTimerEvent *)
{
    angle += 1.0f; // degrees

    // Redraw on a timer
    update();
}

void MyGLWidget::resizeGL(int width, int height)
{
    // Fix viewport to new size
    glViewport(0, 0, width, height);
}

void MyGLWidget::mousePressEvent(QMouseEvent *event)
{
}

void MyGLWidget::mouseMoveEvent(QMouseEvent *event)
{
}
